"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.experiment_router = void 0;
const express_1 = require("express");
const express_async_handler_1 = __importDefault(require("express-async-handler"));
function experiment_router(language, renderPage, _logger) {
    async function experiment(req, res, _next) {
        const templateUrl = req.query.template;
        // TODO: maybe check somewhere else and return a specific page experiment_401.html
        if (!req.user) {
            return renderPage("experiment", language, res, req.user);
        }
        let experiment;
        if (typeof templateUrl === "string") {
            experiment = {
                ...(await req.apiClient.getTemplate(templateUrl)).configuration,
                status: "running",
            };
        }
        if (req.method === "GET") {
            try {
                if (!experiment)
                    experiment = await buildSimpleExperiment(req);
            }
            catch (e) {
                //ignore
            }
        }
        if (req.method === "POST") {
            if (req.body.experiment)
                experiment = JSON.parse(req.body.experiment);
            if (!experiment)
                throw new Error("No experiment provided");
            experiment.status = "running";
            console.log(JSON.stringify(experiment, null, 2));
            const response = await req.apiClient.createExperiment(experiment);
            if (response.status === "setup" && response.url) {
                return experimentSetup(req, res, _next, response);
            }
        }
        try {
            const { pspuGroup, bpuGroup } = await getPspuBpuGroup();
            const pspus = await Promise.all(pspuGroup.devices.map((d) => req.apiClient.getDevice(d.url)));
            const bpus = await Promise.all(bpuGroup.devices.map((d) => req.apiClient.getDevice(d.url)));
            return renderPage("experiment", language, res, req.user, {
                experiment,
                pspus,
                bpus,
            });
        }
        catch {
            return renderPage("experiment", language, res, req.user, {
                experiment,
                pspus: [],
                bpus: [],
            });
        }
        async function getPspuBpuGroup() {
            const devices = await req.apiClient.listDevices();
            const deviceGroups = devices.filter((d) => d.type === "group");
            const pspuGroupUrl = deviceGroups.find((d) => d.name.toLowerCase() === "pspu")?.url;
            const bpuGroupUrl = deviceGroups.find((d) => d.name.toLowerCase() === "bpu")?.url;
            if (!pspuGroupUrl) {
                throw new Error("Could not find pspu group");
            }
            if (!bpuGroupUrl) {
                throw new Error("Could not find bpu group");
            }
            const pspuGroup = await req.apiClient.getDevice(pspuGroupUrl);
            if (pspuGroup.type !== "group") {
                throw new Error("Device is not a group");
            }
            const bpuGroup = await req.apiClient.getDevice(bpuGroupUrl);
            if (bpuGroup.type !== "group") {
                throw new Error("Device is not a group");
            }
            return { pspuGroup, bpuGroup };
        }
    }
    async function experimentSetup(req, res, _next, experiment) {
        const display = req.query.display ?? "link";
        if (experiment.status !== "setup") {
            throw new Error("Experiment is not in setup phase");
        }
        const instances = [];
        for (const device of experiment.instantiatedDevices ?? []) {
            const deviceDetails = await req.apiClient.getDevice(device.instanceOf);
            if (deviceDetails.type === "edge instantiable" &&
                device.url &&
                device.token &&
                deviceDetails.codeUrl) {
                instances.push({
                    codeUrl: deviceDetails.codeUrl,
                    instanceUrl: device.url,
                    deviceToken: device.token,
                });
            }
        }
        //await req.apiClient.updateExperiment(experiment.url!, { devices: experiment.devices })
        return renderPage("experiment-setup", language, res, req.user, {
            experiment,
            instances,
            display
        });
    }
    async function runExperiment(req, res, _next) {
        const resp = await req.apiClient.updateExperiment(req.query.url, {
            status: "running",
        });
        res.send(resp);
        //res.redirect(303, '/' + language + '/index.html');
    }
    const router = (0, express_1.Router)();
    router.get("/experiment", (0, express_async_handler_1.default)(experiment));
    router.post("/experiment", (0, express_async_handler_1.default)(experiment));
    router.get("/run-experiment", (0, express_async_handler_1.default)(runExperiment));
    return router;
}
exports.experiment_router = experiment_router;
const ecpServiceDescription = [
    {
        serviceType: "http://api.goldi-labs.de/serviceTypes/electrical",
        serviceId: "electrical",
        serviceDirection: "prosumer",
        interfaces: [
            { interfaceType: "gpio", availableSignals: { gpio: ["_ANY_"] } },
        ],
    },
    {
        serviceId: "webcam",
        serviceType: "http://api.goldi-labs.de/serviceTypes/webcam",
        serviceDirection: "consumer",
    },
    {
        serviceId: "file",
        serviceType: "https://api.goldi-labs.de/serviceTypes/file",
        serviceDirection: "producer",
    },
];
async function buildSimpleExperiment(req) {
    const pspu = req.query.pspu;
    const bpu = req.query.bpu;
    const ecp = "https://api.goldi-labs.de/devices/cc1de37e-1a6a-4470-affd-12eb41a3231e";
    const devices = [
        { device: pspu, role: "pspu" },
        { device: bpu, role: "bpu" },
        { device: ecp, role: "ecp" },
    ].filter(({ device: url }) => url !== undefined && url !== "");
    const roles = devices.map((d) => ({
        name: d.role,
    }));
    const roleServices = await Promise.all(devices.map(({ device: url, role }) => req.apiClient.getDevice(url).then((device) => ({
        role,
        services: role === "ecp"
            ? ecpServiceDescription
            : device.services,
    }))));
    const serviceConfigurations = [];
    const serviceTypes = new Set();
    roleServices
        .flatMap(({ services }) => services.map(({ serviceType }) => serviceType))
        .forEach((t) => serviceTypes.add(t));
    console.log({ serviceTypes });
    for (const serviceType of serviceTypes) {
        let participants = [];
        for (const { role, services } of roleServices) {
            for (const service of services.filter((s) => s.serviceType === serviceType)) {
                participants.push({
                    role,
                    serviceId: service.serviceId,
                    description: service,
                });
            }
        }
        if (participants.length >= 2) {
            if (serviceType === "http://api.goldi-labs.de/serviceTypes/electrical") {
                const rDI = (s) => {
                    try {
                        return s.replace("#", "");
                    }
                    catch (e) {
                        return s;
                    }
                }; // remove _DRIVEN_ from signal name
                const interfaces = participants.flatMap((p) => {
                    if (p.description.interfaces)
                        return p.description.interfaces.map((i) => ({
                            role: p.role,
                            interface: i,
                        }));
                    else
                        return [];
                });
                const gpioInterfaces = interfaces.filter((i) => i.interface.interfaceType === "gpio");
                const buses = gpioInterfaces.flatMap((i) => {
                    const isDriven = ["inout", "out"].includes(i.interface.direction);
                    if (i.role === "ecp")
                        return [];
                    if (isDriven) {
                        return i.interface.availableSignals.gpio
                            .filter((s) => s.length > 4)
                            .map((s) => ["#" + s]);
                    }
                    else {
                        return i.interface.availableSignals.gpio
                            .filter((s) => s.length > 4)
                            .map((s) => [s]);
                    }
                });
                for (const i of gpioInterfaces) {
                    if (i.role === "ecp")
                        continue;
                    const isDriven = ["inout", "out"].includes(i.interface.direction);
                    const freeSignals = i.interface.availableSignals.gpio.filter((s) => s.length <= 4);
                    for (const bus of buses) {
                        const signal = freeSignals.shift();
                        const isBusDriven = bus.some((s) => s.startsWith("#"));
                        if (signal) {
                            if (isDriven && !isBusDriven) {
                                bus.push("#" + signal);
                            }
                            else {
                                bus.push(signal);
                            }
                        }
                    }
                }
                let id = 0;
                for (const participant of participants) {
                    if (participant.role === "ecp") {
                        participant.config = {
                            interfaces: buses.map((bus) => ({
                                interfaceId: (++id).toString(),
                                interfaceType: "gpio",
                                signals: { gpio: bus.map(rDI).join(" / ") },
                                busId: rDI(bus[0]),
                                direction: bus.some((s) => s.startsWith("#"))
                                    ? "in"
                                    : "inout",
                                driver: participant.role,
                            })),
                        };
                    }
                    else {
                        const interfaces = [participant].flatMap((p) => {
                            if (p.description.interfaces)
                                return p.description.interfaces.map((i) => ({
                                    role: p.role,
                                    interface: i,
                                }));
                            else
                                return [];
                        });
                        const gpioInterfaces = interfaces.filter((i) => i.interface.interfaceType === "gpio");
                        const signals = gpioInterfaces.flatMap((i) => i.interface.availableSignals.gpio.map((s) => {
                            const isDriven = ["inout", "out"].includes(i.interface.direction);
                            if (isDriven) {
                                return "#" + s;
                            }
                            else {
                                return s;
                            }
                        }));
                        const mappedSignals = signals.filter((s) => buses.find((b) => b.map(rDI).includes(rDI(s))));
                        // check if signals are driven in bus:
                        const mappedBusSignals = mappedSignals.map((s) => buses
                            .find((b) => b.map(rDI).includes(rDI(s)))
                            .find((bs) => rDI(bs) === rDI(s)));
                        console.log({ signals, mappedBusSignals, buses });
                        participant.config = {
                            interfaces: mappedBusSignals.map((signal) => ({
                                interfaceId: (++id).toString(),
                                interfaceType: "gpio",
                                signals: { gpio: rDI(signal) },
                                busId: rDI(buses.find((b) => b.map(rDI).includes(rDI(signal)))[0]),
                                direction: signal.startsWith("#") ? "out" : "in",
                                driver: participant.role,
                            })),
                        };
                    }
                }
            }
            else {
                participants = participants.map((p) => ({ ...p, config: {} }));
            }
            participants = participants.map((p) => ({
                ...p,
                description: undefined,
            }));
            serviceConfigurations.push({
                serviceType,
                configuration: {},
                participants,
            });
        }
    }
    return { status: "created", devices, roles, serviceConfigurations };
}
//# sourceMappingURL=experiment.js.map